import os
import sys
import traceback
import json

import ida_idaapi
import idaapi
import idc
import idautils
import ida_diskio

from sms_ida_server import sms_server

try:
    from twilio.rest import Client

except ImportError as e:
    print "Python ImportError: Unable to import twilio! Please install twilio to use SMS_IDA"
    exit(1)

try:
    from flask import Flask, request

except ImportError as e:
    print "Python ImportError: Unable to import flask! Please install flask to use SMS_IDA"
    exit(1)


PLUGIN_VERSION = "1.0"
IDAVERSIONS = "IDA PRO 7.2+"
AUTHORS = "edward@redballoonsecurity.com"
DATE = "2019"

def banner():
    banner_tile = "SMS IDA v{} - (c) {} - {} | {}".format(PLUGIN_VERSION, AUTHORS, DATE,
                                                          IDAVERSIONS)

    print("---[{}]---\n".format(banner_tile))

# print banner on load
banner()

class SmsIdaHandler(idaapi.action_handler_t):
    def __init__(self):
        super(SmsIdaHandler, self)

    @staticmethod
    def activate(ctx):
        pass

    @staticmethod
    def update(ctx):
        print "HANDLER UPDATE"
        return idaapi.AST_ENABLE_ALWAYS

def PLUGIN_ENTRY():
    return SmsIda()


class SmsIda(idaapi.plugin_t):
    flags = idaapi.PLUGIN_FIX

    comment = ""
    help = "SMS Ida - Reverse engineer from your phone, anywhere"
    wanted_name = "SMS Ida"
    wanted_hotkey = ""

    def __init__(self):
        self._account_sid = None
        self._auth_token = None
        self._from_number = None

    def init(self):
        # called by Ida on load
        try:
            pass

        except Exception as e:
            form = idaapi.get_current_tform()
            traceback.print_exc()
            pass

        return ida_idaapi.PLUGIN_KEEP


    def term(self):
        pass

    def run(self, arg=0):

        traceback.print_stack(file=sys.stdout)

        expr = ""

        print idc.eval_idc(expr)

        twilio_auth_path = os.path.join(ida_diskio.get_user_idadir(), 'plugins',
                                        "sms_ida_server", "twilio_auth.json")
        self.load_twilio_auth(twilio_auth_path)

        sms_server.run(debug=False)


    def send_sms(self, phone_number, msg):
        if self._account_sid is None or self._auth_token is None or self._from_number is None:
            print "Error: Twilio auth no initialized."
            return

        client = Client(self._account_sid, self._auth_token)

        message = client.messages.create(body=msg, from_=self._from_number, to=phone_number)

        return message.sid

    def receive_sms(self):
        pass

    def load_twilio_auth(self, filepath):

        try:
            with open(filepath, 'r') as f:
                twilio_auth = json.load(f)
        except IOError as e:
            print "Error opening {}".format(filepath)
            exit(1)

        self._account_sid = twilio_auth['account_sid']
        self._auth_token = twilio_auth['auth_token']
        self._from_number = twilio_auth['from_number']

    def update(self, handler, ctx):
        print "SMS IDA UPDATE"
        return idaapi.AST_ENABLE_ALWAYS
